% Codes for the paper of 'Autonomous Flame Detection in Videos With a 
% Dirichlet Process Gaussian Mixture Color Model' published in 
% IEEE Transactions on Industrial Informatics.
% If you use the codes please cite the above paper in your work
% Copyright 2018 Zhenglin Li
% Matlab implementation for illustrative purposes.

%% 
% clear all; clc;
%%  Preprocessing of the training data
% DataDir = '/path/to/flame/images/dir/';  % Database downloaded from http://cfdb.univ-corse.fr/
% files = dir(DataDir);
% trainD = [];
% RT = 115;
% start_ind = 3; stp = 20; end_ind = 202;  % image index for traning colour model
% for i = start_ind :stp: end_ind
%     rgbimg = imread(strcat(fullfile(DataDir,files(i).name),'/',files(i).name(3:5),'_rgb','.png'));
%     gt = imread(strcat(fullfile(DataDir,files(i).name),'/',files(i).name(3:5),'_gt','.png'));
%     
%     rgbimg = downsample(rgbimg,2); % Downsample the training images
%     rgbimg = permute((downsample(permute(rgbimg,[2 1 3]),2)), [2 1 3]);
% %     figure(1); 
% %     subplot(1,3,1);imshow(rgbimg);
%     img_r = rgbimg(:,:,1);
%     img_g = rgbimg(:,:,2); 
%     img_b = rgbimg(:,:,3);
%     
%     gt = downsample(gt,2);
%     gt = (downsample(gt',2))';
% %     subplot(1,3,2);imshow(gt); title('gt original');
% 
%     gt= (img_r > RT)&gt;  % discard mismarked pixels 
% %     subplot(1,3,3);imshow(gt); title('gt processed');     
%     flmpixel_r = img_r(gt);
%     flmpixel_g = img_g(gt);
%     flmpixel_b = img_b(gt);
%     trainD = cat(1,trainD,[flmpixel_r,flmpixel_g,flmpixel_b]);
% end
% 
% % ind = (trainD(:,1)>=trainD(:,2));
% % ind(trainD(:,2)<=trainD(:,3))=0;
% % ind(trainD(:,1)<115) = 0;
% % trainD = trainD(ind,:); 
% trainD = double(trainD');
% % tmp = sum(trainD,1);
% % for i = 1:size(trainD,2)
% %     trainD(:,i)=trainD(:,i)/tmp(i);
% % end
% save('Trained_Model/TrainData10.mat','trainD');

%%
load 'Trained_Model/TrainData10.mat';
[d,n] = size(trainD);
mu = mean(trainD,2);
trainD_norm = bsxfun(@minus,trainD,mu); 
s = sum(trainD_norm(:).^2)/(d*n); 

opt.alpha = 0.01; % Concentration parameter
opt.kappa = 1;
opt.m = mu;
opt.nu = d; % Degree of freedom,equal to the dimention of data
opt.S = s*eye(d); % Initialisation of \sigma

[y,model,weights] = mixGaussGb(trainD, opt); % model training
figure;
plotClass(trainD,y);
% save('Trained_Model/DPM_10_alpha_0.01.mat','y','model','weights');

% post-processing of the trained model
clr_dim = length(model{1}.m_); %The dimension of the features
clusternum = length(weights);  %The cluster number
Mu = zeros(clusternum,clr_dim); %The means of all clusters
Sigma = zeros(clr_dim,clr_dim,clusternum); %The covariances of all clusters
W = zeros(clusternum,1); %The nomalised weights of all clusters

%Discarding clusters with small weights and calculating covariance matrix for each cluster
t = 1; % The count of remaining clusters
for i = 1:clusternum
    if weights(i)<0.001
        continue;
    end
    Mu(t,:)=(model{i}.m_);
 %   Sigma(:,:,i)=inv(inv((model{i}.U_)'*model{i}.U_)*model{i}.nu_);
    Sigma(:,:,t) = cov((trainD(:,y == i))'); % Using sample covariances to approximate covariance matrices
    W(t)=weights(i);
    t = t+1;
end
W = W/sum(W);
ind = find(W);
W = W(ind);
Mu = Mu(ind,:);
Sigma = Sigma(:,:,ind);
clear t;

% Caluculate probability of each pixel by the proposed colour model
GMM = gmdistribution(Mu,Sigma,W);
save('Trained_Model/DPGMM_10_alpha_001.mat', 'GMM');