% Codes for the paper of 'Autonomous Flame Detection in Videos With a 
% Dirichlet Process Gaussian Mixture Color Model' published in 
% IEEE Transactions on Industrial Informatics.
% If you use the codes please cite the above paper in your work
% Copyright 2018 Zhenglin Li
% Matlab implementation for illustrative purposes.

%%
%dbstop if warning
% clear all; close all; clc;
Allvideodir = 'path/of/videos';
files = dir(Allvideodir);
objnum = length(files);

%%
% parameters of optical flow saliency map
MagApf = 100;  % Amplifying factor of magnitude of optical flow

%% DPGMM colour model
load('../Trained_Model/DPGMM_10_alpha_001.mat', 'GMM'); % loading colour model
clrmodel.name = 'DPGMMRGB';
clrmodel.GMM = GMM;

%% wavelet transform parameters
rcdlen = 16;

%% Threshlods and parameters**********************************
SalThres = 0.21;
AreaThres = 25;
salrcdlen = 5;

%%
for vidind = 3:objnum
    if(files(vidind).isdir)
        continue;
    end
%     if vidind == 14
%         disp 'stop';
%     end
    disp(['File number:' ,num2str(vidind)]);
    videofile = files(vidind).name;
    filename =strcat(Allvideodir,videofile);
    input = VideoReader(filename);
    vidframe = read(input);
    framenum = input.numberOfFrames; 
    hgt = input.Height;
    wdh = input.Width; 
    pnum = hgt * wdh;  
    start = 1;  % starting frame
    step = 1;
    fps = input.FrameRate;
    % load ground truth, variable name 'firelabel'
    load (strcat('/path/framewise/groundtruth/',videofile(1:end-4),'.mat'));  
    resultspath = './Results/'; 
    %***********************************************************************
    % output settings
    outName = fullfile('Results',[videofile(1:end-4) '_results' '.avi']);
    vidOut = VideoWriter(outName);
    vidOut.FrameRate = fps;
    open(vidOut);
 
    %%
    % wavelet transform parameters
    Rrcd = zeros(hgt,wdh,rcdlen);
    
    %% optical flow method and initialization
    objHS = opticalFlowHS;  % Works well
    flowHS = estimateFlow(objHS, rgb2gray(vidframe(:, :, :, start)));
    %% Record of saliency map for smoothing
    Salrcd = zeros(hgt,wdh,salrcdlen);
    %%   
    flag = zeros(framenum,1);
    tp = 0; tn = 0; fp = 0; fn = 0;
    %%
    Runt = 0; % Timer
    tic;
    for i = start:step:framenum
        disp(['frame number:' ,num2str(i)]);
        frameRGB = vidframe(:, :, :, i);
        frameGray = rgb2gray(frameRGB);
        Rrcd = circshift(Rrcd,[0,0,1]);
        Rrcd(:,:,1) = frameRGB(:,:,1);    
        
    %    Compute optical flow
        flowHS = estimateFlow(objHS, frameGray);

    %      Compute saliency map
        tmp = {frameGray, MagApf*flowHS.Magnitude};
        [salMatHS, salMatHSind]=saliencyMeasure(tmp,12);
%         salMatHSavg = (salMatHSind{1}+salMatHSind{2})/2; 
        salMatHSavg = salMatHSind{2}; % Optical flow saliency map
        Salrcd = circshift(Salrcd,[0,0,1]); % Optical flow saliency map sequence update
        Salrcd(:,:,1) = salMatHSavg; %       
        
        if (i-start+1) < rcdlen * step + 1 
            continue;
        end
        salMat_smooth = mean(Salrcd,3);
        
         % color model
        firecolor = FireColorMask(frameRGB,clrmodel); 
        salMatHS_color = salMat_smooth;
        salMatHS_color(firecolor == 0) = 0; % Motion saliency map processed by color model
        
        meanG = mean(frameGray(:));

        binresult = zeros(hgt,wdh);
        binresult(salMatHS_color>SalThres)=1; % Saliency threshold
        binresult(frameGray <= meanG) = 0; % Mean grayvalue threshold

        % erosion and dilation
        B=[0 1 0
           1 1 1
           0 1 0];
        se = strel('arbitrary', B);
        binresult = imdilate(binresult,se);
        binresult = imerode(binresult,se);
        bin_close = binresult;

        %divide candidate regions
        [roi_Lab,roi_num] = bwlabel(binresult);
        if(roi_num == 0)
            disp('no candidate fire region');
            flag(i) = 0; % No fire detected
            if firelabel(i) == 0 % Compared with frame-wise groundtruth
                tn = tn + 1;
            else
                fn = fn + 1;
            end
            output = frameRGB;
            writeVideo(vidOut,im2uint8(output)); % Output video
            continue;
        end
        s = regionprops(roi_Lab,'Area','PixelList');
        clear roi_Lab;
        validroi = roi_num;

        % temporal wavelet analysis
        zpnum = 0; %zero point number
        for rind = 1 : hgt
            for cind = 1 : wdh
                if binresult(rind,cind) == 1
                    [cA,cD] = dwt(reshape(Rrcd(rind,cind,:),[1,rcdlen]),[0.25,0.5,0.25],[-0.25,0.5,-0.25]);
                    cD(and(cD >= -2, cD <= 2)) = 0; % Small wavelet values are treated as 0
          %          [ccA,ccD] = dwt(cA,[0.25,0.5,0.25],[-0.25,0.5,-0.25]);
                    for tmpi = 1 : rcdlen/2
                        if cD(tmpi)*cD(tmpi + 1) <= 0
                            if cD(tmpi) == 0 && cD(tmpi + 1) == 0
                                continue;
                            end
                            zpnum = zpnum + 1;
                        end
                    end
                end
                if zpnum < 5
                    binresult(rind,cind) = 0; % Pixels with fewer than 5 crossings are discarded
                    zpnum = 0;
                    continue;
                end
                zpnum = 0;
            end
        end
        clear rind; clear cind; clear tmpi;

        for ind = 1 : roi_num  % Area rule of each candidate region
            if sum(s(ind).Area) <= AreaThres
                validroi = validroi - 1;
                continue;
            end
        end
        clear ind; clear s;

        output = frameRGB;
        if sum(sum(binresult)) < 20 % Pixel number rule of whole frame
            validroi = 0;
        else
            tmpr = output(:,:,1);
            tmpg = output(:,:,2);
            tmpb = output(:,:,3);
            tmpr(binresult ~= 0) = 255;
            tmpg(binresult ~= 0) = 0;
            tmpb(binresult ~= 0) = 0;
            output(:,:,1) = tmpr;
            output(:,:,2) = tmpg;
            output(:,:,3) = tmpb;
        end
      
        figure(1);
        imshow(frameRGB);
        figure(6);
        imshow(binresult);

    %%
    %************Results measurement*****************
    if validroi ~= 0 && firelabel(i) == 0
       flag(i) = 1;  
       fp = fp + 1;
       disp 'False positive';
    elseif validroi ~= 0 && firelabel(i) == 1
        flag(i) = 1;
        tp = tp + 1;
        disp 'True positive';
    elseif validroi == 0 && firelabel(i) == 0
        flag(i) = 0;
        tn = tn + 1;
        disp 'True negative';
    elseif validroi == 0 && firelabel(i) == 1
        flag(i) = 0;
        fn = fn + 1;
        disp 'False negative';
    end
     writeVideo(vidOut,im2uint8(output));
    end
    Runt = Runt + toc; 
    
    save(strcat(resultspath,videofile(1:end-4),'.mat'),'SalThres','AreaThres','flag');
    if (tp + fn) ~= 0
        TPR = tp/(tp + fn) %true positive
        Recall = tp/(tp + fn)
        save(strcat(resultspath,videofile(1:end-4),'.mat'),'TPR','Recall','-append');
    end
    if (fp + tn) ~= 0
        TNR = tn/(fp + tn) %true negative
        save(strcat(resultspath,videofile(1:end-4),'.mat'),'TNR','-append');
    end
    if (tp + fp) ~= 0
        Precision = tp/(tp + fp)
        save(strcat(resultspath,videofile(1:end-4),'.mat'),'Precision','-append');
    end

    if (2 * tp + fp + fn) ~= 0
        Fscore = 2 * tp /(2 * tp + fp + fn)
        save(strcat(resultspath,videofile(1:end-4),'.mat'),'Fscore','-append');
    end

    save(strcat(resultspath,videofile(1:end-4),'.mat'),'tp','tn','fp','fn','-append');
    close(vidOut);
end